@@ -4,7 +4,7 @@
  * Created Date: 27/05/2021
  * Author: Shun Suzuki
  * -----
- * Last Modified: 05/06/2021
+ * Last Modified: 28/07/2021
  * Modified By: Shun Suzuki (suzuki@hapis.k.u-tokyo.ac.jp)
  * -----
  * Copyright (c) 2021 Hapis Lab. All rights reserved.
@@ -12,9 +12,11 @@
  */
 
 mod sine;
+mod sine_legacy;
 mod sine_pressure;
 mod r#static;
 
 pub use r#static::Static;
 pub use sine::Sine;
+pub use sine_legacy::SineLegacy;
 pub use sine_pressure::SinePressure;
