@@ -4,7 +4,7 @@
  * Created Date: 27/05/2021
  * Author: Shun Suzuki
  * -----
- * Last Modified: 18/06/2021
+ * Last Modified: 14/10/2021
  * Modified By: Shun Suzuki (suzuki@hapis.k.u-tokyo.ac.jp)
  * -----
  * Copyright (c) 2021 Hapis Lab. All rights reserved.
@@ -12,7 +12,7 @@
  */
 
 use anyhow::Result;
-use autd3_core::modulation::Modulation;
+use autd3_core::{hardware_defined, modulation::Modulation};
 use autd3_traits::Modulation;
 
 /// Static amplitude
@@ -20,7 +20,8 @@ use autd3_traits::Modulation;
 pub struct Static {
     buffer: Vec<u8>,
     sent: usize,
-    sampling_freq_div: u16,
+    sampling_freq_div: usize,
+    built: bool,
 }
 
 impl Static {
@@ -40,6 +41,7 @@ impl Static {
             buffer: vec![duty; 1],
             sent: 0,
             sampling_freq_div: 10,
+            built: false,
         }
     }
 
